module.exports = function (point, vs) {
    // ray-casting algorithm based on
    // http://www.ecse.rpi.edu/Homepages/wrf/Research/Short_Notes/pnpoly.html
    
    var x = point[0], y = point[1];
    
    var inside = false;
    for (var i = 0, j = vs.length - 1; i < vs.length; j = i++) {
        var xi = vs[i][0], yi = vs[i][1];
        var xj = vs[j][0], yj = vs[j][1];
        
        var intersect = ((yi > y) != (yj > y))
            && (x < (xj - xi) * (y - yi) / (yj - yi) + xi);
        if (intersect) inside = !inside;
    }

    //Determine whether the mouse is clicked on the border
    if(!inside) {
      //error range
      var range = 2;

      for (var i = 0, j = vs.length - 1; i < vs.length; j = i++) {
        var xi = vs[i][0], yi = vs[i][1];
        var xj = vs[j][0], yj = vs[j][1];

        var cross = (xj - xi) * (x - xi) + (yj - yi) * (y - yi);
        var d2 = (xj - xi) * (xj - xi) + (yj - yi) * (yj - yi);

        var r = cross / d2;
        var px = xi + (xj - xi) * r;
        var py = yi + (yj - yi) * r;
        //Determine whether the distance is less than the error
        if (Math.sqrt((x - px) * (x - px) + (py - y) * (py - y)) <= range) {
          inside = true;
          //Determine whether the point is outside the line segment
          if (cross <= 0 || cross >= d2) {
            inside = false
          }
          return inside
        }

      }
    }


    return inside;
};
